
/*
 *  returns the dayOfYear for the autumn equinox
 *  accurate for years 2022 through 2040.
 *  Close enough for other years (i.e. 265)
 */
int autumnEquinoxDay(int year)
{
  int autumnEquinoxDays[] = {
        265, 265, 266, 265, 265, 265, 266, 265, 265, 265,
        266, 265, 265, 265, 266, 265, 265, 265, 266
    };

  int tableStartYear = 2022;
  int tableEndYear = 2040;
  if(year>tableEndYear  || year < tableStartYear) return 265;
  return autumnEquinoxDays[year-tableStartYear];
}


 int dayOfYear(long time)
 {
     // Number of days in each month (non-leap year)
     const int daysInMonth[] = {0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};

     int year = Time.year(time);
     int month = Time.month(time);
     int day = Time.day(time);

    // Check for leap year (leap years have 366 days)
     int isLeapYear = ((year % 4 == 0 && year % 100 != 0) || (year % 400 == 0));
     // Add up days until this month
     int dayOfYear = 0;
     for(int i=1;i<month;i++)
     {
       dayOfYear += daysInMonth[i];
     }
     dayOfYear = dayOfYear + day;

     if(isLeapYear && month > 2) dayOfYear++;
     return dayOfYear;
 }

String metricMonthName(int metricMonth)
  {
   char* republicanMonths[] = {
         "Vendemiaire", "Brumaire", "Frimaire", "Nivose", "Pluviose", "Ventose",
         "Messidor", "Thermidor", "Fructidor", "Prairial", "Floreal", "Messidor",
         "Sans-culottide"
     };
     String monthName = "Unknown";
     if(metricMonth >= 0  && metricMonth < 12)
     {
       monthName = String(republicanMonths[metricMonth]);
     }
     return monthName;
}

String metricDayName(int metricDayOfMonth)
  {
   const char *dayNames[] = {
        "Primidi", "Duodi", "Tridi", "Quartidi", "Quintidi", "Sextidi", "Septidi", "Octidi", "Nonidi", "Decadi"
    };
     int dayNumber = metricDayOfMonth%10;
     String dayName = String(dayNames[dayNumber-1]);
     return dayName;
}
/*
   * The Revolutionary calendar year 1 starts with the fall equinox
   * in 1792.  New year is the fall equinox.  Then there are 12 months of 3 10
   * day weeks each.  Then either 5 or 6 extra days at the end to round out the
   * year.
   */
void metricDateFromTime(double _time, metricDateStruct* _date)
  {
    long time = (long) _time;
    int calendarYearZero = 1792;
    int year = Time.year(time);
    int thisYearAutumnDay = autumnEquinoxDay(year);
    int lastYearAutumnDay = autumnEquinoxDay(year-1);
    int metricYear = year-calendarYearZero;
    int day = dayOfYear(time);
    int metricDayOfYear = day + (365-lastYearAutumnDay) + 1;
    if(day > thisYearAutumnDay)
      {
        metricYear++;
        metricDayOfYear = day - thisYearAutumnDay + 1;
      }
    int metricDayOfMonth = (metricDayOfYear%30)+1;
    _date->year = metricYear;
    _date->month = metricDayOfYear/30;
    _date->day = metricDayOfMonth;
  }

String romanNumerals(long value) //format value as roman numerals.
{

  if (value <= 0) {
      return "error";
  }

    char* roman[] = {"M", "CM", "D", "CD", "C", "XC", "L", "XL", "X", "IX", "V", "IV", "I"};
    int values[] = {1000, 900, 500, 400, 100, 90, 50, 40, 10, 9, 5, 4, 1};
    String answer = "";
    for (int i = 0; value > 0; i++) {
        while (value >= values[i]) {
            answer = answer + roman[i];
            value -= values[i];
        }
    }
    return answer;
}


String Formatters::metricTime(double _now)  //format value as metric time hh:mm:ss
{
  long todaysSeconds = ((long)_now) % (24*3600);
  long metricSeconds = (long) todaysSeconds / 0.864;  //the metric second is shorter
  int hours = metricSeconds/(100*100);
  int minutes = (metricSeconds % (100*100))/100;
  int seconds = metricSeconds%100;
  String time = String::format("%2i:%02i:%02i",hours,minutes,seconds);
  return time;
};

/*
 * The Revolutionary calendar year 1 starts with the fall equinox
 * in 1792.  New year is the fall equinox.  Then there are 12 months of 3 10
 * day weeks each.  Then either 5 or 6 extra days at the end to round out the
 * year.
 */
String Formatters::metricDate(double _time) //format value as metric date
{
  struct metricDateStruct dateStruct;
  metricDateFromTime(_time,&dateStruct);
  String monthString = metricMonthName(dateStruct.month);
  return String(dateStruct.day) + " " + monthString;
}

String Formatters::metricDay(double _time)
{
  struct metricDateStruct dateStruct;
  metricDateFromTime(_time,&dateStruct);
  return  metricDayName(dateStruct.day);
}

String Formatters::metricYear(double _value)
{
  struct metricDateStruct dateStruct;
  metricDateFromTime(_value,&dateStruct);
  int year = dateStruct.year;
  return romanNumerals(year);

}
